/* $Id: rex_prot.x,v 1.3 2000/01/05 07:29:55 dm Exp $ */

/*
 * This file was written by David Mazieres.  Its contents is
 * uncopyrighted and in the public domain.  Of course, standards of
 * academic honesty nonetheless prevent anyone in research from
 * falsely claiming credit for this work.
 */

%#include "sfs_prot.h"

typedef string ttypath<63>;
typedef string utmphost<16>;

/* Note, a successful PTYD_PTY_ALLOC result is accompanied by a file
 * descriptor for the master side of the pty. */
union pty_alloc_res switch (int err) {
 case 0:
   ttypath path;
 default:
   void;
};

program PTYD_PROG {
	version PTYD_VERS {
		void
		PTYD_NULL (void) = 0;

		pty_alloc_res
		PTYD_PTY_ALLOC (utmphost) = 1;

		int
		PTYD_PTY_FREE (ttypath) = 2;
	} = 1;
} = 344431;

/* Must be accompanied by a previously negotiated authuint */
struct rexd_spawn_arg {
  sfs_kmsg kmsg;
  string command<>;
};

struct rexd_spawn_resok {
  sfs_kmsg kmsg;
};

union rexd_spawn_res switch (sfsstat err) {
 case SFS_OK:
   rexd_spawn_resok resok;
 default:
   void;
};

struct rexd_attach_arg {
  sfs_hash sessid;
  sfs_seqno seqno;
  sfs_hash newsessid;
};

#if 0
struct rexd_attach_resok {
};

union rexd_attach_res switch (sfsstat err) {
 case SFS_OK:
   rexd_attach_resok resok;
 default:
   void;
};
#else
typedef sfsstat rexd_attach_res;
#endif

struct rex_sesskeydat {
  sfs_msgtype type;		/* = SFS_KSC or SFS_KCS */
  sfs_seqno seqno;
  sfs_secret sshare;		/* Server's share of session key */
  sfs_secret cshare;		/* Client's share of session key */
};

program REXD_PROG {
	version REXD_VERS {
		void
		REXD_NULL (void) = 0;

		rexd_spawn_res
		REXD_SPAWN (rexd_spawn_arg) = 1;

		rexd_attach_res
		REXD_ATTACH (rexd_attach_arg) = 2;
	} = 1;
} = 344424;

program REXCTL_PROG {
	version REXCTL_VERS {
		void
		REXCTL_NULL (void) = 0;

		/* REXCTL_ACCEPT is preceeded by a file descriptor */
		void
		REXCTL_CONNECT (sfs_sessinfo) = 1;
	} = 1;
} = 344426;

struct rex_payload {
  unsigned channel;
  opaque data<>;
};

typedef string rex_progarg<>;
typedef rex_progarg rex_cmd<>;

const REXCH_READ = 1;
const REXCH_WRITE = 2;
const REXCH_RDWR = 3;
struct rex_channel {
  int flags;
  int fd<>;
};

struct rex_mkchannel_arg {
  rex_cmd av;
  rex_channel chv<>;
};

struct rex_mkchannel_resok {
  int chv<>;
};

union rex_mkchannel_res switch (sfsstat err) {
 case SFS_OK:
   rex_mkchannel_resok resok;
 default:
   void;
};

program REX_PROG {
	version REX_VERS {
		void
		REX_NULL (void) = 0;

		void
		REX_DATA (rex_payload) = 1;

		rex_mkchannel_res
		REX_MKCHANNEL (rex_mkchannel_arg) = 2;
	} = 1;
} = 344428;

program REXCB_PROG {
	version REXCB_VERS {
		void
		REXCB_NULL (void) = 0;

		void
		REXCB_DATA (rex_payload) = 1;
	} = 1;
} = 344429;
